@@ -1,9 +1,6 @@
 import threading
 from .packages.special.main import COMMANDS
-try:
-    from collections import OrderedDict
-except ImportError:
-    from .packages.ordereddict import OrderedDict
+from collections import OrderedDict
 
 from .sqlcompleter import SQLCompleter
 from .sqlexecute import SQLExecute
@@ -16,9 +13,8 @@ class CompletionRefresher(object):
         self._completer_thread = None
         self._restart_refresh = threading.Event()
 
-    def refresh(self, executor, callbacks):
-        """
-        Creates a SQLCompleter object and populates it with the relevant
+    def refresh(self, executor, callbacks, completer_options=None):
+        """Creates a SQLCompleter object and populates it with the relevant
         completion suggestions in a background thread.
 
         executor - SQLExecute object, used to extract the credentials to connect
@@ -26,14 +22,20 @@ class CompletionRefresher(object):
         callbacks - A function or a list of functions to call after the thread
                     has completed the refresh. The newly created completion
                     object will be passed in as an argument to each callback.
+        completer_options - dict of options to pass to SQLCompleter.
+
         """
+        if completer_options is None:
+            completer_options = {}
+
         if self.is_refreshing():
             self._restart_refresh.set()
             return [(None, None, None, 'Auto-completion refresh restarted.')]
         else:
-            self._completer_thread = threading.Thread(target=self._bg_refresh,
-                                                      args=(executor, callbacks),
-                                                      name='completion_refresh')
+            self._completer_thread = threading.Thread(
+                target=self._bg_refresh,
+                args=(executor, callbacks, completer_options),
+                name='completion_refresh')
             self._completer_thread.setDaemon(True)
             self._completer_thread.start()
             return [(None, None, None,
@@ -42,8 +44,8 @@ class CompletionRefresher(object):
     def is_refreshing(self):
         return self._completer_thread and self._completer_thread.is_alive()
 
-    def _bg_refresh(self, sqlexecute, callbacks):
-        completer = SQLCompleter(smart_completion=True)
+    def _bg_refresh(self, sqlexecute, callbacks, completer_options):
+        completer = SQLCompleter(**completer_options)
 
         # Create a new pgexecute method to popoulate the completions.
         e = sqlexecute
