@@ -6,25 +6,20 @@ import os
 from os.path import exists
 import struct
 import sys
+
 from configobj import ConfigObj, ConfigObjError
+from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
+from cryptography.hazmat.backends import default_backend
+
 try:
     basestring
 except NameError:
     basestring = str
-try:
-    from Crypto.Cipher import AES
-except ImportError:
-    AES = None
 
 
-class CryptoError(Exception):
-    """
-    Exception to signal about pycrypto not available.
-    """
-    pass
-
 logger = logging.getLogger(__name__)
 
+
 def log(logger, level, message):
     """Logs message to stderr if logging isn't initialized."""
 
@@ -33,6 +28,7 @@ def log(logger, level, message):
     else:
         print(message, file=sys.stderr)
 
+
 def read_config_file(f):
     """Read a config file."""
 
@@ -40,7 +36,7 @@ def read_config_file(f):
         f = os.path.expanduser(f)
 
     try:
-        config = ConfigObj(f, interpolation=False)
+        config = ConfigObj(f, interpolation=False, encoding='utf8')
     except ConfigObjError as e:
         log(logger, logging.ERROR, "Unable to parse line {0} of config file "
             "'{1}'.".format(e.line_number, f))
@@ -53,6 +49,7 @@ def read_config_file(f):
 
     return config
 
+
 def read_config_files(files):
     """Read and merge a list of config files."""
 
@@ -66,6 +63,7 @@ def read_config_files(files):
 
     return config
 
+
 def write_default_config(source, destination, overwrite=False):
     destination = os.path.expanduser(destination)
     if not overwrite and exists(destination):
@@ -73,6 +71,7 @@ def write_default_config(source, destination, overwrite=False):
 
     shutil.copyfile(source, destination)
 
+
 def get_mylogin_cnf_path():
     """Return the path to the login path file or None if it doesn't exist."""
     mylogin_cnf_path = os.getenv('MYSQL_TEST_LOGIN_FILE')
@@ -89,6 +88,7 @@ def get_mylogin_cnf_path():
         return mylogin_cnf_path
     return None
 
+
 def open_mylogin_cnf(name):
     """Open a readable version of .mylogin.cnf.
 
@@ -101,7 +101,7 @@ def open_mylogin_cnf(name):
     try:
         with open(name, 'rb') as f:
             plaintext = read_and_decrypt_mylogin_cnf(f)
-    except (OSError, IOError):
+    except (OSError, IOError, ValueError):
         logger.error('Unable to open login path file.')
         return None
 
@@ -111,6 +111,7 @@ def open_mylogin_cnf(name):
 
     return TextIOWrapper(plaintext)
 
+
 def read_and_decrypt_mylogin_cnf(f):
     """Read and decrypt the contents of .mylogin.cnf.
 
@@ -125,8 +126,6 @@ def read_and_decrypt_mylogin_cnf(f):
     :return: the decrypted login path file
     :rtype: io.BytesIO or None
     """
-    if AES is None:
-        raise CryptoError('pycrypto is not available.')
 
     # Number of bytes used to store the length of ciphertext.
     MAX_CIPHER_STORE_LEN = 4
@@ -154,8 +153,8 @@ def read_and_decrypt_mylogin_cnf(f):
             return None
     rkey = struct.pack('16B', *rkey)
 
-    # Create a cipher object using the key.
-    aes_cipher = AES.new(rkey, AES.MODE_ECB)
+    # Create a decryptor object using the key.
+    decryptor = _get_decryptor(rkey)
 
     # Create a bytes buffer to hold the plaintext.
     plaintext = BytesIO()
@@ -169,24 +168,9 @@ def read_and_decrypt_mylogin_cnf(f):
 
         # Read cipher_len bytes from the file and decrypt.
         cipher = f.read(cipher_len)
-        pplain = aes_cipher.decrypt(cipher)
-
-        try:
-            # Determine pad length.
-            pad_len = ord(pplain[-1:])
-        except TypeError:
-            # ord() was unable to get the value of the byte.
-            logger.warning('Unable to remove pad.')
+        plain = _remove_pad(decryptor.update(cipher))
+        if plain is False:
             continue
-
-        if pad_len > len(pplain) or len(set(pplain[-pad_len:])) != 1:
-            # Pad length should be less than or equal to the length of the
-            # plaintext. The pad should have a single unqiue byte.
-            logger.warning('Invalid pad found in login path file.')
-            continue
-
-        # Get rid of pad.
-        plain = pplain[:-pad_len]
         plaintext.write(plain)
 
     if plaintext.tell() == 0:
@@ -196,6 +180,7 @@ def read_and_decrypt_mylogin_cnf(f):
     plaintext.seek(0)
     return plaintext
 
+
 def str_to_bool(s):
     """Convert a string value to its corresponding boolean value."""
     if isinstance(s, bool):
@@ -212,3 +197,29 @@ def str_to_bool(s):
         return False
     else:
         raise ValueError('not a recognized boolean value: %s'.format(s))
+
+
+def _get_decryptor(key):
+    """Get the AES decryptor."""
+    c = Cipher(algorithms.AES(key), modes.ECB(), backend=default_backend())
+    return c.decryptor()
+
+
+def _remove_pad(line):
+    """Remove the pad from the *line*."""
+    pad_length = ord(line[-1:])
+    try:
+        # Determine pad length.
+        pad_length = ord(line[-1:])
+    except TypeError:
+        # ord() was unable to get the value of the byte.
+        logger.warning('Unable to remove pad.')
+        return False
+
+    if pad_length > len(line) or len(set(line[-pad_length:])) != 1:
+        # Pad length should be less than or equal to the length of the
+        # plaintext. The pad should have a single unqiue byte.
+        logger.warning('Invalid pad found in login path file.')
+        return False
+
+    return line[:-pad_length]
