@@ -1,13 +1,19 @@
 import logging
 import pymysql
 import sqlparse
-from .packages import connection, special
+from .packages import special
 from pymysql.constants import FIELD_TYPE
 from pymysql.converters import (convert_mysql_timestamp, convert_datetime,
-        convert_timedelta, convert_date)
+                                convert_timedelta, convert_date, conversions,
+                                decoders)
 
 _logger = logging.getLogger(__name__)
 
+FIELD_TYPES = decoders.copy()
+FIELD_TYPES.update({
+    FIELD_TYPE.NULL: type(None)
+})
+
 class SQLExecute(object):
 
     databases_query = '''SHOW DATABASES'''
@@ -17,6 +23,7 @@ class SQLExecute(object):
     version_query = '''SELECT @@VERSION'''
 
     version_comment_query = '''SELECT @@VERSION_COMMENT'''
+    version_comment_query_mysql4 = '''SHOW VARIABLES LIKE "version_comment"'''
 
     show_candidates_query = '''SELECT name from mysql.help_topic WHERE name like "SHOW %"'''
 
@@ -41,6 +48,7 @@ class SQLExecute(object):
         self.local_infile = local_infile
         self.ssl = ssl
         self._server_type = None
+        self.connection_id = None
         self.connect()
 
     def connect(self, database=None, user=None, password=None, host=None,
@@ -64,18 +72,19 @@ class SQLExecute(object):
             '\tlocal_infile: %r'
             '\tssl: %r',
             database, user, host, port, socket, charset, local_infile, ssl)
-        conv = {
-                FIELD_TYPE.TIMESTAMP: lambda obj: (convert_mysql_timestamp(obj) or obj),
-                FIELD_TYPE.DATETIME: lambda obj: (convert_datetime(obj) or obj),
-                FIELD_TYPE.TIME: lambda obj: (convert_timedelta(obj) or obj),
-                FIELD_TYPE.DATE: lambda obj: (convert_date(obj) or obj),
-                }
+        conv = conversions.copy()
+        conv.update({
+            FIELD_TYPE.TIMESTAMP: lambda obj: (convert_mysql_timestamp(obj) or obj),
+            FIELD_TYPE.DATETIME: lambda obj: (convert_datetime(obj) or obj),
+            FIELD_TYPE.TIME: lambda obj: (convert_timedelta(obj) or obj),
+            FIELD_TYPE.DATE: lambda obj: (convert_date(obj) or obj),
+        })
 
-        conn = connection.connect(database=db, user=user, password=password,
+        conn = pymysql.connect(database=db, user=user, password=password,
                 host=host, port=port, unix_socket=socket,
                 use_unicode=True, charset=charset, autocommit=True,
                 client_flag=pymysql.constants.CLIENT.INTERACTIVE,
-                cursorclass=connection.Cursor, local_infile=local_infile,
+                local_infile=local_infile,
                 conv=conv, ssl=ssl)
         if hasattr(self, 'conn'):
             self.conn.close()
@@ -90,6 +99,8 @@ class SQLExecute(object):
         self.socket = socket
         self.charset = charset
         self.ssl = ssl
+        # retrieve connection id
+        self.reset_connection_id()
 
     def run(self, statement):
         """Execute the sql in the database and return the results. The results
@@ -107,7 +118,6 @@ class SQLExecute(object):
         # want to save them all together.
         if statement.startswith('\\fs'):
             components = [statement]
-
         else:
             components = sqlparse.split(statement)
 
@@ -119,31 +129,40 @@ class SQLExecute(object):
             if sql.endswith('\\G'):
                 special.set_expanded_output(True)
                 sql = sql[:-2].strip()
+
+            cur = self.conn.cursor()
             try:   # Special command
                 _logger.debug('Trying a dbspecial command. sql: %r', sql)
-                cur = self.conn.cursor()
                 for result in special.execute(cur, sql):
                     yield result
             except special.CommandNotFound:  # Regular SQL
-                yield self.execute_normal_sql(sql)
+                _logger.debug('Regular sql statement. sql: %r', sql)
+                cur.execute(sql)
+                while True:
+                    yield self.get_result(cur)
 
-    def execute_normal_sql(self, split_sql):
-        _logger.debug('Regular sql statement. sql: %r', split_sql)
-        cur = self.conn.cursor()
-        num_rows = cur.execute(split_sql)
+                    # PyMySQL returns an extra, empty result set with stored
+                    # procedures. We skip it (rowcount is zero and no
+                    # description).
+                    if not cur.nextset() or (not cur.rowcount and cur.description is None):
+                        break
+
+    def get_result(self, cursor):
+        """Get the current result's data from the cursor."""
         title = headers = None
 
-        # cur.description is not None for queries that return result sets, e.g.
-        # SELECT or SHOW.
-        if cur.description is not None:
-            headers = [x[0] for x in cur.description]
+        # cursor.description is not None for queries that return result sets,
+        # e.g. SELECT or SHOW.
+        if cursor.description is not None:
+            headers = [x[0] for x in cursor.description]
             status = '{0} row{1} in set'
         else:
             _logger.debug('No rows in result.')
             status = 'Query OK, {0} row{1} affected'
-        status = status.format(num_rows, '' if num_rows == 1 else 's')
+        status = status.format(cursor.rowcount,
+                               '' if cursor.rowcount == 1 else 's')
 
-        return (title, cur if cur.description else None, headers, status)
+        return (title, cursor if cursor.description else None, headers, status)
 
     def tables(self):
         """Yields table names"""
@@ -182,7 +201,7 @@ class SQLExecute(object):
             _logger.debug('Show Query. sql: %r', self.show_candidates_query)
             try:
                 cur.execute(self.show_candidates_query)
-            except pymysql.OperationalError as e:
+            except pymysql.DatabaseError as e:
                 _logger.error('No show completions due to %r', e)
                 yield ''
             else:
@@ -194,7 +213,7 @@ class SQLExecute(object):
             _logger.debug('Users Query. sql: %r', self.users_query)
             try:
                 cur.execute(self.users_query)
-            except pymysql.OperationalError as e:
+            except pymysql.DatabaseError as e:
                 _logger.error('No user completions due to %r', e)
                 yield ''
             else:
@@ -208,9 +227,19 @@ class SQLExecute(object):
             _logger.debug('Version Query. sql: %r', self.version_query)
             cur.execute(self.version_query)
             version = cur.fetchone()[0]
-            _logger.debug('Version Comment. sql: %r', self.version_comment_query)
-            cur.execute(self.version_comment_query)
-            version_comment = cur.fetchone()[0].lower()
+            if version[0] == '4':
+                _logger.debug('Version Comment. sql: %r',
+                              self.version_comment_query_mysql4)
+                cur.execute(self.version_comment_query_mysql4)
+                version_comment = cur.fetchone()[1].lower()
+                if isinstance(version_comment, bytes):
+                    # with python3 this query returns bytes
+                    version_comment = version_comment.decode('utf-8')
+            else:
+                _logger.debug('Version Comment. sql: %r',
+                              self.version_comment_query)
+                cur.execute(self.version_comment_query)
+                version_comment = cur.fetchone()[0].lower()
 
         if 'mariadb' in version_comment:
             product_type = 'mariadb'
@@ -221,3 +250,16 @@ class SQLExecute(object):
 
         self._server_type = (product_type, version)
         return self._server_type
+
+    def get_connection_id(self):
+        if not self.connection_id:
+            self.reset_connection_id()
+        return self.connection_id
+
+    def reset_connection_id(self):
+        # Remember current connection id
+        _logger.debug('Get current connection id')
+        res = self.run('select connection_id()')
+        for title, cur, headers, status in res:
+            self.connection_id = cur.fetchone()[0]
+        _logger.debug('Current connection id: %s', self.connection_id)
